/**
 * (?<CallExpressionName>useStaticQuery
 *   (?<TypeTemplate><
 *     (?<TypeArgument>\S*)
 *   >)?
 * )
 * \([\s\S]*?
 * graphql
 * (?<TemplateLiteral>`\s*?
 *   (?<QueryDefinitionStart>query
 *     (?<QueryName>\S*)
 *     [^{]?\{
 *   )
 *   [^`]*?
 * `)
 */
const STATIC_QUERY_HOOK_REGEXP = /(?<CallExpressionName>useStaticQuery(?<TypeTemplate><(?<TypeArgument>\S*)>)?)\([\s\S]*?graphql(?<TemplateLiteral>`\s*?(?<QueryDefinitionStart>query (?<QueryName>\S*)[^\{]\{)[^`]*?`)/g;
const STATIC_QUERY_HOOK_REPLACER = (
  substring: string,
  ...args: Array<{
    CallExpressionName: string;
    QueryDefinitionStart: string;
    QueryName: string;
    TemplateLiteral: string;
    TypeArgument: string;
    TypeTemplate: string;
  }>
): string => {
  const groups = args.pop();

  if (groups == null) {
    return substring;
  }

  return substring.replace(
    groups.CallExpressionName,
    `useStaticQuery<${groups.QueryName}Query>`,
  );
};

/**
 * (?<JsxTagOpening><StaticQuery
 *   (?<TagTypeTemplate><
 *     (?<TagTypeArgument>\S+)
 *   >)?
 * )
 * [\s\S]+?
 * query={
 * [\s\S]*?
 * graphql
 * (?<TemplateLiteral>`\s*?
 *   (?<QueryDefinitionStart>query
 *     (?<QueryName>\S*)
 *     [^{]?\{
 *   )
 *   [^`]*?
 * `)
 */
const STATIC_QUERY_COMPONENT_REGEXP = /(?<JsxTagOpening><StaticQuery(?<TagTypeTemplate><(?<TagTypeArgument>\S+)>)?)[\s\S]+?query=\{[\s\S]*?graphql(?<TemplateLiteral>`\s*?(?<QueryDefinitionStart>query (?<QueryName>\S*)[^\{]?\{)[^`]*`)/g;
const STATIC_QUERY_COMPONENT_REPLACER = (
  substring: string,
  ...args: Array<{
    JsxTagOpening: string;
    TagTypeTemplate: string;
    TagTypeArgument: string;
    TemplateLiteral: string;
    QueryDefinitionStart: string;
    QueryName: string;
  }>
): string => {
  const groups = args.pop();

  if (groups == null) {
    return substring;
  }

  return substring.replace(
    groups.JsxTagOpening,
    `<StaticQuery<${groups.QueryName}Query>`,
  );
};

export const fixUseStaticQuery = (input: string): string =>
  input.replace(STATIC_QUERY_HOOK_REGEXP, STATIC_QUERY_HOOK_REPLACER);
export const fixStaticQuery = (input: string): string =>
  input.replace(STATIC_QUERY_COMPONENT_REGEXP, STATIC_QUERY_COMPONENT_REPLACER);
